@@ -1,5 +1,5 @@
 /* This file is part of GNU Dico.
-   Copyright (C) 1998-2000, 2008, 2010, 2012 Sergey Poznyakoff
+   Copyright (C) 1998-2018 Sergey Poznyakoff
 
    GNU Dico is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -99,9 +99,9 @@ dicod_load_module(dicod_module_instance_t *inst)
 {
     struct wordsplit ws;
     int rc;
-
+    
     if (wordsplit(inst->command, &ws, WRDSF_DEFFLAGS)) {
-	dico_log(L_ERR, rc, _("cannot parse command line `%s': %s"),
+	dico_log(L_ERR, 0, _("cannot parse command line `%s': %s"),
 		 inst->command, wordsplit_strerror (&ws));
 	return 1;
     }
@@ -597,4 +597,41 @@ dicod_define_word_all(dico_stream_t stream, const char *word)
 		   "250 Command complete",
 		   print_definitions, NULL, "define");
 }
-	    
+
+int
+dicod_module_test(int argc, char **argv)
+{
+    int i;
+    dicod_module_instance_t inst;
+    struct dico_database_module *pmod;    
+    
+    /* Split arguments in two parts: unit test arguments and
+       optional module initialization arguments */
+    for (i = 0; i < argc; i++)
+	if (strcmp(argv[i], "--") == 0) {
+	    break;
+	}
+
+    if (i == 0)
+	dico_die(EX_UNAVAILABLE, L_ERR, 0, _("no module name"));
+    
+    memset(&inst, 0, sizeof(inst));
+    if (i < argc) {
+	argv[i] = argv[0];
+	if (dicod_load_module0(&inst, argc - i, argv + i))
+	    return 1;
+	argv[i] = NULL;
+    } else {
+	char *null_argv[2];
+	null_argv[0] = argv[0];
+	null_argv[1] = NULL;
+	if (dicod_load_module0(&inst, 1, null_argv))
+	    return 1;
+    }
+
+    pmod = inst.module;
+    if (pmod->dico_version <= 1 || !pmod->dico_run_test)
+	dico_die(EX_UNAVAILABLE, L_ERR, 0,
+		 _("module does not define dico_run_test function"));
+    return pmod->dico_run_test(i, argv);
+}
